@@ -2,8 +2,8 @@ def foo()
   $block = Block.new
 end
 
-foo(){i| print "i = ", i, "\n"}
+foo(){|i| print "i = ", i, "\n"}
 $block.call(2)
 
-foo(){i| print "i*2 = ", i*2, "\n"}
+foo(){|i| print "i*2 = ", i*2, "\n"}
 $block.call(2)
