@@ -1,7 +1,7 @@
 # Linked list example
 class MyElem
   # ֥˼ưŪ˸ƤФ᥽å
-  def init_object(item)
+  def initialize(item)
     # @ѿϥ󥹥ѿ(פʤ)
     @data = item
     @next = nil
@@ -54,7 +54,7 @@ class MyList
 end
 
 class Point
-  def init_object(x, y)
+  def initialize(x, y)
     @x = x; @y = y
     self
   end
