@@ -2,7 +2,7 @@
 # using _inspect
 
 class Point
-  def init_object(x, y)
+  def initialize(x, y)
     @x = x; @y = y
     self
   end
